#############################################################################################
##  INTEGRATED POPULATION MODEL FOR SSB FEMALES FROM 2000/01 TO 2021/22
#--------------------------------------------------------------------------------------------
##--LOAD PACKAGES-
library(jagsUI)
library(rjags)
#-----------------
### SET WORKING DIRECTORY
##  CHANGE THIS TO YOUR WORKING DIRECTORY
setwd("C:/.../")
#-----------------
## DATA FEMALE PUPS PIT-TAGGED AT BIRTH  2001 TO 2022
## Observations (O):
## 1 seen as weanling
## 2 seen as pre-breeder
## 3 seen breeder
## 4 not seen
## -------------------------------------------------
##  DATA OF FEMALES TAGGED AS BREEDERS 2001 TO 2022
## Observations (O):
## 1 seen as breeder
## 2 seen as non-breeder
## 3 not seen
## -------------------------------------------------
#   Functions borrowed from Kery and Schaub (2014) to:
##  1. create known latent states z for data input
known.state.ms <- function(ms, notseen){
  state <- ms
  state[state == notseen] <- NA
  for (i in 1:dim(ms)[1]){
    m <- min(which(!is.na(state[i, ])))
    state[i, m] <- NA
  }
  return(state)
}
## generate initial values for unknown z
ms.init.z <- function(ch, f){
  for(i in 1:dim(ch)[1]) 
    ch[i, 1:f[i]] <- NA
  states <- max(ch, na.rm = TRUE)
  known.states <- 1:(states - 1)
  v <- which(ch == states)
  ch[-v] <- NA
  ch[v] <- sample(known.states, length(v), replace = TRUE)
  return(ch)
}
#
##
####----------------------------------------------------------------------------
#### JAGS MODELLING
#### specify BUGS model in JAGS
sink("SSB.IM") 
cat("
  model{
    # Priors
    ## Initial population sizes
    N1[1] ~ dnorm(110, 0.0001)T(0,)
    N2[1] ~ dnorm(87, 0.0001)T(0,)
    NB3[1] ~ dnorm(2.23, 0.0001)T(0, )
    NP3[1] ~ dnorm(71, 0.0001)T(0, )
    NB4[1] ~ dnorm(15, 0.0001)T(0, )
    NP4[1] ~ dnorm(44, 0.0001)T(0, )
    NB5[1] ~ dnorm(12, 0.0001)T(0, )
    NP5[1] ~ dnorm(22, 0.0001)T(0, )
    NB6[1] ~ dnorm(3, 0.0001)T(0, )
    NP6[1] ~ dnorm(13, 0.0001)T(0, )
    NB7[1] ~ dnorm(1.3, 0.0001)T(0, )
    NP7[1] ~ dnorm(6, 0.0001)T(0, )
    NP[1] ~ dnorm(3.5, 0.0001)T(0, )
    NN[1] ~ dnorm(254, 0.0001)T(0, )
    NB[1] ~ dnorm(696, 0.0001)T(0, )
    NI[1] ~ dnorm(0, 0.0001)T(0, )
    #
    ## Sex ratio
    for(i in 1:T){
      sr[i] <- srp[i] / nFp[i]
      srp[i] ~ dbin(pF[i],nFp[i])
    }
    # perinatal survival
    #for(i in 1:T){
    #  phi0[i] ~ dbeta(alphaS[i], betaS[i])
    #}
    ## PRIORS AND CONSTRAINTS
    for(t in 1:(T - 1)){
      ## Adult survival and psiBN with correlated temporal random efffects
      logit(phiB[t]) <- eta.phipsiBN[t, 1]
      logit(phiN[t]) <- eta.phipsiBN[t, 2]
      logit(psiBN[t]) <- eta.phipsiBN[t, 3]
      ## FOR 1ST DATA SET ANALYSIS
      phi.B[t] <- mean.phiB
      ## PRE-BREEDER COMPONENT
      ## phi - epsilon. are temporal random efects
      logit(phi.1[t]) <- mu.1 + epsilon.1[t]
      epsilon.1[t] ~ dnorm(0, tau.1)T(-15, 15)
      logit(phi.2[t]) <- mu.2 + epsilon.2[t]
      epsilon.2[t] ~ dnorm(0, tau.2)T(-15, 15)
      logit(phi.3[t]) <- mu.3 + epsilon.3[t]
      epsilon.3[t] ~ dnorm(0, tau.3)T(-15, 15)
      logit(phi.4[t]) <- mu.4 + epsilon.4[t]
      epsilon.4[t] ~ dnorm(0, tau.4)T(-15, 15)
      logit(phi.5[t]) <- mu.5 + epsilon.5[t]
      epsilon.5[t] ~ dnorm(0, tau.5)T(-15, 15)
      logit(phi.6[t]) <- mu.6 + epsilon.6[t]
      epsilon.6[t] ~ dnorm(0, tau.6)T(-15, 15)
      logit(phi.7[t]) <- mu.7 + epsilon.7[t]
      epsilon.7[t] ~ dnorm(0, tau.7)T(-15, 15)
      logit(phi.8[t]) <- mu.8 + epsilon.8[t]
      epsilon.8[t] ~ dnorm(0, tau.8)T(-15, 15)
      #
      ## alpha (RECRUITMENT)
      logit(alpha.3[t]) <- mu.a3 + epsilon.a3[t]
      epsilon.a3[t] ~ dnorm(0, tau.a3)T(-15, 15)
      logit(alpha.4[t]) <- mu.a4 + epsilon.a4[t]
      epsilon.a4[t] ~ dnorm(0, tau.a4)T(-15, 15)
      logit(alpha.5[t]) <- mu.a5 + epsilon.a5[t]
      epsilon.a5[t] ~ dnorm(0, tau.a5)T(-15, 15)
      logit(alpha.6[t]) <- mu.a6 + epsilon.a6[t]
      epsilon.a6[t] ~ dnorm(0, tau.a6)T(-15, 15)
      logit(alpha.7[t]) <- mu.a7 + epsilon.a7[t]
      epsilon.a7[t] ~ dnorm(0, tau.a7)T(-15, 15)
      #
      ## BREEDING PROBABILITY FOR SKIPPERS
      ## psi NB independent temporal random effects
      logit(psiNB[t]) <- muNB + epsilonNB[t]
      epsilonNB[t] ~ dnorm(0, tauNB)T(-15, 15)
      #
      ## RECAPTURE (p)
      p.P1[t] <- mean.pP1
      p.P2[t] <- mean.pP2
      pB[t] <- mean.pB
      ## temporal random efects on recapture of skippers
      logit(pN[t]) <- mupN + epsilonpN[t]
      epsilonpN[t] ~ dnorm(0, taupN)T(-15, 15)
      ## IMMIGRATION (iota)
      log(iota[t]) <- miota + epsilonIo[t]
      epsilonIo[t] ~ dnorm(0, tau.iota)T(-15, 15)
    }
    ## survival and psiBN
    for(t in 1:(T-1)){
      eta.phipsiBN[t, 1:3] ~ dmnorm(mu.phipsiBN[], Omega[,])
    }
    for(u in 1:3){
      ## Priors for mean mature state-specific survival
      mean.phipsiBN[u] ~ dunif(0, 1)                    
      mu.phipsiBN[u] <- log(mean.phipsiBN[u] / (1-mean.phipsiBN[u]))
    }
    ## Priors for variance-covariance matrix
    Omega[1:3, 1:3] ~ dwish(R[, ], 4)
    Sigma.phipsiBN[1:3, 1:3] <- inverse(Omega[, ])
    #
    ## PREBREEDER SURVIVAL, priors, means and precisions
    mu.1 <- log(mean.phi.1 / (1 - mean.phi.1))
    mean.phi.1 ~ dunif(0, 1)
    tau.1 <- pow(sigma.1, -2)
    sigma.1 ~ dunif(0, 10)
    mu.2 <- log(mean.phi.2 / (1 - mean.phi.2))
    mean.phi.2 ~ dunif(0, 1)
    tau.2 <- pow(sigma.2, -2)
    sigma.2 ~ dunif(0, 10)
    mu.3 <- log(mean.phi.3 / (1 - mean.phi.3))
    mean.phi.3 ~ dunif(0, 1)
    tau.3 <- pow(sigma.3, -2)
    sigma.3 ~ dunif(0, 10)
    mu.4 <- log(mean.phi.4 / (1 - mean.phi.4))
    mean.phi.4 ~ dunif(0, 1)
    tau.4 <- pow(sigma.4, -2)
    sigma.4 ~ dunif(0, 10)
    mu.5 <- log(mean.phi.5 / (1 - mean.phi.5))
    mean.phi.5 ~ dunif(0, 1)
    tau.5 <- pow(sigma.5, -2)
    sigma.5 ~ dunif(0, 10)
    mu.6 <- log(mean.phi.6 / (1 - mean.phi.6))
    mean.phi.6 ~ dunif(0, 1)
    tau.6 <- pow(sigma.6, -2)
    sigma.6 ~ dunif(0, 10)
    mu.7 <- log(mean.phi.7 / (1 - mean.phi.7))
    mean.phi.7 ~ dunif(0, 1)                            # Prior for mean return to breeding
    tau.7 <- pow(sigma.7, -2)
    sigma.7 ~ dunif(0, 10)
    mu.8 <- log(mean.phi.8 / (1 - mean.phi.8))
    mean.phi.8 ~ dunif(0, 1)
    tau.8 <- pow(sigma.8, -2)
    sigma.8 ~ dunif(0, 10)
    mean.phiB ~ dunif(0, 1)     # Prior for mean breeder survival
    #
    ## RECRUITMENT, priors, means and precisions
    mu.a3 <- log(mean.alpha.3 / (1 - mean.alpha.3))
    mean.alpha.3 ~ dunif(0, 1)
    tau.a3 <- pow(sigma.a3, -2)
    sigma.a3 ~ dunif(0, 10)    
    mu.a4 <- log(mean.alpha.4 / (1 - mean.alpha.4))
    mean.alpha.4 ~ dunif(0, 1)
    tau.a4 <- pow(sigma.a4, -2)
    sigma.a4 ~ dunif(0, 10)  
    mu.a5 <- log(mean.alpha.5 / (1 - mean.alpha.5))
    mean.alpha.5 ~ dunif(0, 1)
    tau.a5 <- pow(sigma.a5, -2)
    sigma.a5 ~ dunif(0, 10)  
    mu.a6 <- log(mean.alpha.6 / (1 - mean.alpha.6))
    mean.alpha.6 ~ dunif(0, 1)
    tau.a6 <- pow(sigma.a6, -2)
    sigma.a6 ~ dunif(0, 10)  
    mu.a7 <- log(mean.alpha.7 / (1 - mean.alpha.7))
    mean.alpha.7 ~ dunif(0, 1)
    tau.a7 <- pow(sigma.a7, -2)
    sigma.a7 ~ dunif(0, 10)  
    #
    ## Breeding probability for skippers
    muNB <- log(mean.psiNB / (1 - mean.psiNB))
    ## Prior for mean return to breeding
    mean.psiNB ~ dunif(0, 1)
    tauNB <- pow(sigmaNB, -2)
    sigmaNB ~ dunif(0, 10)
    #
    ## RECAPTURE
    ## Prior for mean recapture young pre-breeders
    mean.pP1 ~ dunif(0, 1)
    ## Prior for mean recapture old pre-breeders
    mean.pP2 ~ dunif(0, 1)
    ## Priors for mean recapture of breeders
    mean.pB ~ dunif(0, 1)
    mupN <- log(mean.pN / (1 - mean.pN))
    ## Prior for mean survival of N
    mean.pN ~ dunif(0, 1)
    taupN <- pow(sigmapN, -2)
    sigmapN ~ dunif(0, 10)
    #
    ## IMMIGRATION RATE priors
    miota ~ dnorm(0, 0.0001)T(-10, 10)
    sigma.iota ~ dunif(0, 10)
    tau.iota <- pow(sigma.iota, -2)
    #
    ## DERIVED PARAMETERS
    ## mean immigration rate
    eiota <- exp(miota)
    for(t in 1:T){
      ## total mature seals
      Ntot[t] <- NBI[t]+NN[t]
      ## recruits
      Nrec[t] <- NB3[t]+NB4[t]+NB5[t]+NB6[t]+NB7[t]
      ## pre-recruits
      Npre[t] <- N1[t]+N2[t]+NP3[t]+NP4[t]+NP5[t]+NP6[t]+NP7[t]+NP[t]
    }
    for(t in 1:T){
      ## total female population size
      NAll[t] <- Ntot[t] + Npre[t]
      ## pregnancy rate
      PR[t] <- NBI[t] / Ntot[t]
    }
    for(t in 1:(T-1)){
      lambda[t] <- Ntot[t+1] / Ntot[t]  # population growth rate
      loglambda[t] <- log(lambda[t])  # log-lambda
    }
    meanlambda <- exp((1 / (T - 1)) * sum(loglambda[1:(T - 1)]))  # geometric mean
    #
    ## PRODUCTIVITY AND RECRUITMENT
    ### system process
    for(t in 2:T){
      mean1[t] <- sr[t-1]*phi0[t-1]*phi.1[t-1]*NBI[t-1]       # weaned pups surviving to age 1
      N1[t] ~ dpois(mean1[t])                                 # 1-year-olds (widges)
      mean2[t] <- phi.2[t-1]*N1[t-1]
      N2[t] ~ dpois(mean2[t])                                 # 2-year-olds (widges)
      meanb3[t] <- phi.3[t-1]*alpha.3[t-1]*N2[t-1]
      NB3[t] ~ dpois(meanb3[t])                               # 3-year-old recruits
      meanp3[t] <- phi.3[t-1]*(1-alpha.3[t-1])*N2[t-1]
      NP3[t] ~ dpois(meanp3[t])                               # 3-year-old pre-breeders
      meanb4[t] <- phi.4[t-1]*alpha.4[t-1]*NP3[t-1]
      NB4[t] ~ dpois(meanb4[t])                               # 4-year-old recruits
      meanp4[t] <- phi.4[t-1]*(1-alpha.4[t-1])*NP3[t-1]
      NP4[t] ~ dpois(meanp4[t])                               # 4-year-old pre-breeders
      meanb5[t] <- phi.5[t-1]*alpha.5[t-1]*NP4[t-1]
      NB5[t] ~ dpois(meanb5[t])                               # 5-year-old recruits
      meanp5[t] <- phi.5[t-1]*(1-alpha.5[t-1])*NP4[t-1]
      NP5[t] ~ dpois(meanp5[t])                               # 5-year-old pre-breeders
      meanb6[t] <- phi.6[t-1]*alpha.6[t-1]*NP5[t-1]
      NB6[t] ~ dpois(meanb6[t])                               # 6-year-old recruits
      meanp6[t] <- phi.6[t-1]*(1-alpha.6[t-1])*NP5[t-1]
      NP6[t] ~ dpois(meanp6[t])                               # 6-year-old pre-breeders
      meanb7[t] <- phi.7[t-1]*alpha.7[t-1]*NP6[t-1]
      NB7[t] ~ dpois(meanb7[t])                               # 7-year-old recruits
      meanp7[t] <- phi.7[t-1]*(1-alpha.7[t-1])*NP6[t-1]
      NP7[t] ~ dpois(meanp7[t])                               # 7-year-old pre-breeders
      meanNP[t] <- NP7[t-1]*phi.8[t-1]                        # Pre-breeders never observed breeding
      NP[t] ~ dpois(meanNP[t])
      meanB[t] <- NBI[t-1]*phiB[t-1]*(1-psiBN[t-1])+phi.3[t-1]*alpha.3[t-1]*N2[t-1]+phi.4[t-1]*alpha.4[t-1]*NP3[t-1]+phi.5[t-1]*alpha.5[t-1]*NP4[t-1]+phi.6[t-1]*alpha.6[t-1]*NP5[t-1]+phi.7[t-1]*alpha.7[t-1]*NP6[t-1]+NN[t-1]*phiN[t-1]*psiNB[t-1]
      NB[t] ~ dpois(meanB[t])                                 # Breeders
      meanN[t] <- NBI[t-1]*phiB[t-1]*psiBN[t-1]+NN[t-1]*phiN[t-1]*(1-psiNB[t-1])        # breeders that skip breeding
      NN[t] ~ dpois(meanN[t])                                 # Non-Breeders
      meanI[t] <- (NBI[t-1]+NN[t-1])*iota[t-1]                # IMMIGRANTS
      NI[t] ~ dpois(meanI[t])
    }
    #
    ## OBSERVATION PROCESS
    ## observed productivity relates directly to the number of observed breeders (NB + NI)
    for(t in 1:T){
      NBI[t] <- NB[t]+NI[t]
      prod[t] ~ dpois(NBI[t])
    }
    ## FIRST CAPTURE-RECAPTURE; FEMAES TAGGED AS NEWBORN PUPS
    ## State-transition and observation matrices
     for(i in 1:nind1){
      ## Define probabilities of state S(t+1) given S(t)
      for(t in f1[i]:(T - 1)){
        ps1[1, i, t, 1] <- 0
        ps1[1, i, t, 2] <- phi.1[t]
        ps1[1, i, t, 3] <- 0
        ps1[1, i, t, 4] <- 0
        ps1[1, i, t, 5] <- 0
        ps1[1, i, t, 6] <- 0
        ps1[1, i, t, 7] <- 0
        ps1[1, i, t, 8] <- 0
        ps1[1, i, t, 9] <- 0        
        ps1[1, i, t,10] <- 1 - phi.1[t] 
        ps1[2, i, t, 1] <- 0
        ps1[2, i, t, 2] <- 0
        ps1[2, i, t, 3] <- phi.2[t]
        ps1[2, i, t, 4] <- 0
        ps1[2, i, t, 5] <- 0
        ps1[2, i, t, 6] <- 0
        ps1[2, i, t, 7] <- 0
        ps1[2, i, t, 8] <- 0
        ps1[2, i, t, 9] <- 0        
        ps1[2, i, t,10] <- 1 - phi.2[t] 
        ps1[3, i, t, 1] <- 0
        ps1[3, i, t, 2] <- 0
        ps1[3, i, t, 3] <- 0
        ps1[3, i, t, 4] <- phi.3[t] * (1 - alpha.3[t])
        ps1[3, i, t, 5] <- 0
        ps1[3, i, t, 6] <- 0
        ps1[3, i, t, 7] <- 0
        ps1[3, i, t, 8] <- 0
        ps1[3, i, t, 9] <- phi.3[t] * alpha.3[t]
        ps1[3, i, t,10] <- 1 - phi.3[t] 
        ps1[4, i, t, 1] <- 0
        ps1[4, i, t, 2] <- 0
        ps1[4, i, t, 3] <- 0
        ps1[4, i, t, 4] <- 0
        ps1[4, i, t, 5] <- phi.4[t] * (1 - alpha.4[t])
        ps1[4, i, t, 6] <- 0
        ps1[4, i, t, 7] <- 0
        ps1[4, i, t, 8] <- 0
        ps1[4, i, t, 9] <- phi.4[t] * alpha.4[t]
        ps1[4, i, t,10] <- 1 - phi.4[t] 
        ps1[5, i, t, 1] <- 0
        ps1[5, i, t, 2] <- 0
        ps1[5, i, t, 3] <- 0
        ps1[5, i, t, 4] <- 0
        ps1[5, i, t, 5] <- 0
        ps1[5, i, t, 6] <- phi.5[t] * (1 - alpha.5[t])
        ps1[5, i, t, 7] <- 0
        ps1[5, i, t, 8] <- 0
        ps1[5, i, t, 9] <- phi.5[t] * alpha.5[t]
        ps1[5, i, t,10] <- 1 - phi.5[t] 
        ps1[6, i, t, 1] <- 0
        ps1[6, i, t, 2] <- 0
        ps1[6, i, t, 3] <- 0
        ps1[6, i, t, 4] <- 0
        ps1[6, i, t, 5] <- 0
        ps1[6, i, t, 6] <- 0
        ps1[6, i, t, 7] <- phi.6[t] * (1 - alpha.6[t])
        ps1[6, i, t, 8] <- 0
        ps1[6, i, t, 9] <- phi.6[t] * alpha.6[t]
        ps1[6, i, t,10] <- 1 - phi.6[t] 
        ps1[7, i, t, 1] <- 0
        ps1[7, i, t, 2] <- 0
        ps1[7, i, t, 3] <- 0
        ps1[7, i, t, 4] <- 0
        ps1[7, i, t, 5] <- 0
        ps1[7, i, t, 6] <- 0
        ps1[7, i, t, 7] <- 0
        ps1[7, i, t, 8] <- phi.7[t] * (1 - alpha.7[t])
        ps1[7, i, t, 9] <- phi.7[t] * alpha.7[t]
        ps1[7, i, t,10] <- 1 - phi.7[t] 
        ps1[8, i, t, 1] <- 0
        ps1[8, i, t, 2] <- 0
        ps1[8, i, t, 3] <- 0
        ps1[8, i, t, 4] <- 0
        ps1[8, i, t, 5] <- 0
        ps1[8, i, t, 6] <- 0
        ps1[8, i, t, 7] <- 0
        ps1[8, i, t, 8] <- 0
        ps1[8, i, t, 9] <- phi.8[t]
        ps1[8, i, t,10] <- 1 - phi.8[t] 
        ps1[9, i, t, 1] <- 0
        ps1[9, i, t, 2] <- 0
        ps1[9, i, t, 3] <- 0
        ps1[9, i, t, 4] <- 0
        ps1[9, i, t, 5] <- 0
        ps1[9, i, t, 6] <- 0
        ps1[9, i, t, 7] <- 0
        ps1[9, i, t, 8] <- 0
        ps1[9, i, t, 9] <- phi.B[t]
        ps1[9, i, t,10] <- 1 - phi.B[t] 
        ps1[10, i, t, 1] <- 0
        ps1[10, i, t, 2] <- 0
        ps1[10, i, t, 3] <- 0
        ps1[10, i, t, 4] <- 0
        ps1[10, i, t, 5] <- 0
        ps1[10, i, t, 6] <- 0
        ps1[10, i, t, 7] <- 0
        ps1[10, i, t, 8] <- 0
        ps1[10, i, t, 9] <- 0
        ps1[10, i, t,10] <- 1
        ## Define probabilities of O(t) given S(t)
        po1[1, i, t, 1] <- 0
        po1[1, i, t, 2] <- 0
        po1[1, i, t, 3] <- 0
        po1[1, i, t, 4] <- 1
        po1[2, i, t, 1] <- 0
        po1[2, i, t, 2] <- p.P1[t]
        po1[2, i, t, 3] <- 0
        po1[2, i, t, 4] <- 1 - p.P1[t]
        po1[3, i, t, 1] <- 0
        po1[3, i, t, 2] <- p.P1[t]
        po1[3, i, t, 3] <- 0
        po1[3, i, t, 4] <- 1 - p.P1[t]
        po1[4, i, t, 1] <- 0
        po1[4, i, t, 2] <- p.P1[t]
        po1[4, i, t, 3] <- 0
        po1[4, i, t, 4] <- 1 - p.P1[t]        
        po1[5, i, t, 1] <- 0
        po1[5, i, t, 2] <- p.P2[t]
        po1[5, i, t, 3] <- 0
        po1[5, i, t, 4] <- 1 - p.P2[t]        
        po1[6, i, t, 1] <- 0
        po1[6, i, t, 2] <- p.P2[t]
        po1[6, i, t, 3] <- 0
        po1[6, i, t, 4] <- 1 - p.P2[t]        
        po1[7, i, t, 1] <- 0
        po1[7, i, t, 2] <- p.P2[t]
        po1[7, i, t, 3] <- 0
        po1[7, i, t, 4] <- 1 - p.P2[t]        
        po1[8, i, t, 1] <- 0
        po1[8, i, t, 2] <- p.P2[t]
        po1[8, i, t, 3] <- 0
        po1[8, i, t, 4] <- 1 - p.P2[t]        
        po1[9, i, t, 1] <- 0
        po1[9, i, t, 2] <- 0
        po1[9, i, t, 3] <- pB[t]
        po1[9, i, t, 4] <- 1 - pB[t]        
        po1[10, i, t, 1] <- 0
        po1[10, i, t, 2] <- 0
        po1[10, i, t, 3] <- 0
        po1[10, i, t, 4] <- 1
      } #t
    } #i
    ##   
    ## LIKELIHOOD
    for(i in 1:nind1){
      # Define latent state at first capture
      z1[i, f1[i]] <- y1[i, f1[i]]
      for(t in (f1[i] + 1):T){
        # State process: draw S(t) given S(t - 1)
        z1[i, t] ~ dcat(ps1[z1[i, t - 1], i, t - 1, ])
        # Observation process: draw O(t) given S(t)
        y1[i, t] ~ dcat(po1[z1[i, t], i, t - 1, ])
      } #t
    } #i
    ## SECOND CAPTURE-RECAPTURE DATA SET; LIFTED FEMALES
    ## State-transition and observation matrices
    for(i in 1:nind2){  
      # Define probabilities of state S(t + 1) given S(t)
      for (t in f2[i]:(T - 1)){
        ps2[1, i, t, 1] <- phiB[t] * (1 - psiBN[t])
        ps2[1, i, t, 2] <- phiB[t] * psiBN[t]
        ps2[1, i, t, 3] <- 1 - phiB[t]
        ps2[2, i, t, 1] <- phiN[t] * psiNB[t]
        ps2[2, i, t, 2] <- phiN[t] * (1 - psiNB[t])
        ps2[2, i, t, 3] <- 1 - phiN[t]
        ps2[3, i ,t, 1] <- 0
        ps2[3, i, t, 2] <- 0
        ps2[3, i, t, 3] <- 1
        #    
        ## Define probabilities of O(t) given S(t)
        po2[1, i, t, 1] <- pB[t]
        po2[1, i, t, 2] <- 0
        po2[1, i, t, 3] <- 1 - pB[t]
        po2[2, i, t, 1] <- 0
        po2[2, i, t, 2] <- pN[t]
        po2[2, i, t, 3] <- 1 - pN[t]
        po2[3, i, t, 1] <- 0
        po2[3, i, t, 2] <- 0
        po2[3, i, t, 3] <- 1
      } #t
    } #i
    # Likelihood 
    for(i in 1:nind2){
      # Define latent state at first capture
      z2[i, f2[i]] <- y2[i, f2[i]]
      for(t in (f2[i] + 1):T){
        # State process: draw S(t) given S(t - 1)
        z2[i, t] ~ dcat(ps2[z2[i, t - 1], i, t - 1, ])
        # Observation process: draw O(t) given S(t)
        y2[i, t] ~ dcat(po2[z2[i, t], i, t - 1, ])
      } #t
    } #i
    #
    ## Poisson GAMM for adjustment of counts
    eta <- X %*% bb ## linear predictor
    for(i in 1:nn){
      mu[i] <-  exp(eta[i] + eps[i])
      eps[i] ~ dnorm(0, tau)
    } ## expected response
    for(i in 1:nn){
      yy[i] ~ dpois(mu[i])
    } ## response 
    ## random effects priors
    tau <- pow(sd, -2)
    sd ~ dunif(0, 5)
    ## Parametric effect priors CHECK tau=1/52^2 is appropriate!
    for(i in 1:1){
      bb[i] ~ dnorm(0,0.00041)
    }
    ## prior for s(x0)... 
    for(i in c(2:9)){
      bb[i] ~ dnorm(0, lambda[1])
    }
    for(i in c(10)){
      bb[i] ~ dnorm(0, lambda[2])
    }
    ## smoothing parameter priors CHECK...
    for(i in 1:2) {
      lambda2[i] ~ dgamma(.05,.005)
      rho[i] <- log(lambda2[i])
    }
    ## OTHER DERIVED QUANTITIES
    for(j in 1:7){
      NMat.nc[j] <- (NBI[9] +  NN[9]) / mu[32+j]
      NBI.nc[j] <- NBI[9] / mu[32 + j]
    }    
  }
  ", fill = TRUE)
sink()
##
#
## SOURCE DATA
rCH1 <- readRDS("rCH1.Rds") # PIT-TAGGED CAPTURE HISTORIES
rCH2 <- readRDS("rCH2.Rds") # TAGGED AS BREEDERS CAPTURE HISTORIES
prod <- readRDS("prod.Rds") # PUP PRODUCTION
pFem <- readRDS("pFem.Rds") # PROPORTION OF FEMALE PUPS
nFp <- prod
f1 <- readRDS("f1.Rds")
f2 <- readRDS("f2.Rds")
phi0 <- readRDS("phi0.Rds")
yy <- readRDS("yy.Rds")
nn <- readRDS("nn.Rds")
X <- readRDS("X.Rds")
z1 <- readRDS("z1.Rds")
##
jags.data <- list(y1 = rCH1, y2 = rCH2, prod = prod, pF = pFem, nFp = nFp, f1 = f1, f2 = f2, 
  T = dim(rCH1)[2], nind1 = dim(rCH1)[1], nind2 = dim(rCH2)[1], 
  phi0 = phi0, z2 = known.state.ms(rCH2, 3), R = matrix(c(5, 0, 0, 0, 1, 0, 0, 0, 1), ncol = 3),
  yy = yy, nn = nn, X = X)
#
inits <- function()
  list(Omega = matrix(c(1, 0, 0, 0, 1, 0, 0, 0, 1), ncol = 3), 
       mean.psiNB = runif(1, 0, 1), sigmaNB=runif(1, 0, 1),
       mean.pB = runif(1, 0, 1), mean.pN = runif(1, 0, 1), sigmapN = runif(1, 0, 1), mean.pP1 = runif(1, 0, 1), mean.pP2 = runif(1, 0, 1),
       mean.phi.1=runif(1, 0, 1), mean.phi.2=runif(1, 0, 1),mean.phi.3=runif(1, 0, 1),mean.phi.4=runif(1, 0, 1),mean.phi.5=runif(1, 0, 1),mean.phi.6=runif(1, 0, 1),mean.phi.7=runif(1, 0, 1),mean.phi.8=runif(1, 0, 1),mean.phiB = runif(1, 0, 1),
       mean.alpha.3=runif(1, 0, 1),mean.alpha.4=runif(1, 0, 1),mean.alpha.5=runif(1, 0, 1),mean.alpha.6=runif(1, 0, 1),mean.alpha.7=runif(1, 0, 1),
       z1 = z1, z2 = ms.init.z(rCH2, f2),
       miota = rnorm(1, 0.1, 0.5),
       sigma.iota = runif(1, 0.1, 10),
       N1 = round(runif(dim(rCH1)[2], 10, 110), 0),
       N2 = round(runif(dim(rCH1)[2], 10, 87), 0),
       NB3 = round(runif(dim(rCH1)[2], 1, 3), 0),
       NP3 = round(runif(dim(rCH1)[2], 5, 71), 0),
       NB4 = round(runif(dim(rCH1)[2], 2, 15), 0),
       NP4 = round(runif(dim(rCH1)[2], 5, 44), 0),
       NB5 = round(runif(dim(rCH1)[2], 2, 12), 0),
       NP5 = round(runif(dim(rCH1)[2], 5, 22), 0),
       NB6 = round(runif(dim(rCH1)[2], 1, 5), 0),
       NP6 = round(runif(dim(rCH1)[2], 2, 15), 0),
       NB7 = round(runif(dim(rCH1)[2], 1, 3), 0),
       NP7 = round(runif(dim(rCH1)[2], 2, 10), 0),
       NP = round(runif(dim(rCH1)[2], 3, 5), 0),
       NN = round(runif(dim(rCH1)[2], 10, 100), 0),
       NB = round(runif(dim(rCH1)[2], 200, 700), 0),
       NI = round(runif(dim(rCH1)[2], 10, 100), 0),
       bb = jd$jags.ini$b,
       lambda2 = jd$jags.ini$lambda)
#
## Parameters monitored
parameters <- c("phi.1","phi.2","phi.3","phi.4","phi.5","phi.6","phi.7","phi.8","phi.B","phiB", "phiN",
                "alpha.3","alpha.4","alpha.5","alpha.6","alpha.7","psiBN", "psiNB", "iota", "eiota",
                "Sigma.phipsiBN", "mean.psiNB", "mean.pP1", "mean.pP2", "mean.pB","pN", "mean.pN",
                "sigmaNB", "sigmapN", "sigma.iota", 
                "Ntot", "NAll", "Nrec", "Npre", "NB", "NN", "NI", "NBI",
                "N1", "N2", "NB3", "NP3", "NB4", "NP4", "NB5", "NP5", "NB6", "NP6", "NB7", "NP7","NP",
                "PR", "lambda", "loglambda", "meanlambda", "NMat.nc", "NBI.nc", "bb", "rho", "mu", "sd", "eps")
#
## MCMC settings
ni <- 50000
nt <- 6
nb <- 20000
nc <- 3
#
##
### Run the BUGS model
sta <- Sys.time()
out.test <- jags(jags.data, inits, parameters, "SSB.IM", n.chains = nc, n.thin =  nt, n.iter = ni, n.burnin = nb)
Sys.time() - sta
# Time difference of 9.967746 days !!!!
##
## WAIC
samples <- rjags::jags.samples(out.test$model, c("WAIC", "deviance"), type = "mean", n.iter = ni, n.burnin = nb, n.thin = nc)
samples$p_waic <- samples$WAIC
samples$waic <- samples$deviance + samples$p_waic
(tmp <- sapply(samples, sum))
#      WAIC   deviance     p_waic       waic 
#  763.2301  9883.0630   763.2301 10646.2930 
(waic_temp <- round(c(deviance = tmp[["deviance"]], waic = tmp[["waic"]], p_waic = tmp[["p_waic"]]), 2))
# deviance     waic   p_waic 
#  9883.06 10646.29   763.23 
#
################################################################################